
uniform mat4 uWorldMatrix;
uniform mat4 uWorldViewProjMatrix;
uniform vec4 uEyePosition;

attribute vec3 aPosition;
attribute vec3 aNormal;
attribute vec2 aTexCoord;

varying vec3 vNormal;
varying vec3 vWorldPosition;
varying vec2 vTexCoord;
varying vec3 vViewDirection;

void main() {
    vec4 position = vec4(aPosition, 1.0);
	gl_Position = uWorldViewProjMatrix * position;

	vec4 normal = vec4(aNormal, 0.0);
	vNormal = (uWorldMatrix * normal).xyz;

	vec3 worldPosition = (uWorldMatrix * position).xyz;
	vWorldPosition = worldPosition;

	vTexCoord = aTexCoord;

	vViewDirection = worldPosition - uEyePosition.xyz;
}

// #SplitMarker

precision mediump float;

uniform sampler2D sBaseMap;
uniform sampler2D sTrackLampsMap;
uniform sampler2D sMineLampMap;
uniform sampler2D sIrradianceMap;

const vec4 cBlackColor = vec4(0.0, 0.0, 0.0, 1.0);
const float cLimRz = 0.01;
const float cMineLimRy = 0.1;
const float cTrackLimRy = 0.7;
const float cTrackLimDimRy = 0.3;
const vec3 cLightNormal = vec3(0.0, -0.643, 0.766); // altitude 50 degrees
const vec3 cTrackCeilingPos = vec3(-20.0, -10.0, 25.0);
const vec2 cTrackCeilingDim = vec2(190.0, 20.0);
const vec3 cMineCeilingPos = vec3(180.56, -8.0, 4.9);
const vec2 cMineCeilingDim = vec2(8.0, 16.0);
const vec2 cIrradiancePos = vec2(-2.4, -8.5);
const vec2 cIrradianceDim = vec2(204.0, 17.0);

varying vec3 vNormal;
varying vec3 vWorldPosition;
varying vec2 vTexCoord;
varying vec3 vViewDirection;

void main() {
	vec3 normal = normalize(vNormal);
	vec3 viewDirection = normalize(vViewDirection);

	vec4 baseColor = texture2D(sBaseMap, vTexCoord);
	float diffuseValue = 0.5 * (dot(cLightNormal, normal) + 1.0);
	vec4 diffuse = baseColor * diffuseValue;

	vec2 irradPos = vWorldPosition.yx;
	vec2 irradUV = (irradPos - cIrradiancePos) / cIrradianceDim;
	float irradiance = texture2D(sIrradianceMap, irradUV).r;

	vec4 specular = cBlackColor;
	vec3 reflection = reflect(viewDirection, normal);
	if (reflection.z > cLimRz) {
		float mineT = (cMineCeilingPos.z - vWorldPosition.z) / reflection.z;
		vec2 mineP = vWorldPosition.yx + mineT * reflection.yx;
		if (mineP.x > cMineCeilingPos.x) {
			// Mine lamp
			if (reflection.y < cMineLimRy) {
				float intensity = min((cMineLimRy - reflection.y) / cMineLimRy, 1.0);
				vec2 mineUV = (mineP - cMineCeilingPos.xy) / cMineCeilingDim;
				specular = intensity * texture2D(sMineLampMap, mineUV);
			}
		} else {
			// Track lamps
			if (reflection.y < cTrackLimRy) {
				float intensity = min((cTrackLimRy - reflection.y) / cTrackLimDimRy, 1.0);
				float trackT = (cTrackCeilingPos.z - vWorldPosition.z) / reflection.z;
				vec2 trackP = vWorldPosition.yx + trackT * reflection.yx;
				vec2 trackUV = (trackP - cTrackCeilingPos.xy) / cTrackCeilingDim;
				specular = intensity * texture2D(sTrackLampsMap, trackUV);
			}
		}
	}

	gl_FragColor = irradiance * (diffuse + specular);
}
