
uniform mat4 uWorldMatrix;
uniform mat4 uWorldViewProjMatrix;
uniform vec4 uEyePosition;

attribute vec3 aPosition;
attribute vec3 aNormal;
attribute vec2 aTexCoord;

varying vec3 vNormal;
varying vec3 vWorldPosition;
varying vec2 vTexCoord;
varying vec3 vViewDirection;

void main() {
    vec4 position = vec4(aPosition, 1.0);
	gl_Position = uWorldViewProjMatrix * position;

	vec4 normal = vec4(aNormal, 0.0);
	vNormal = (uWorldMatrix * normal).xyz;

	vec3 worldPosition = (uWorldMatrix * position).xyz;
	vWorldPosition = worldPosition;

	vTexCoord = aTexCoord;

	vViewDirection = worldPosition - uEyePosition.xyz;
}

// #SplitMarker

precision mediump float;

uniform sampler2D sBaseMap;
uniform sampler2D sIrradianceMap;

const vec3 cLightNormal = vec3(0.0, -0.643, 0.766); // altitude 50 degrees
const float cSpecularPower = 25.0;
const vec2 cIrradiancePos = vec2(-2.4, -8.5);
const vec2 cIrradianceDim = vec2(204.0, 17.0);

varying vec3 vNormal;
varying vec3 vWorldPosition;
varying vec2 vTexCoord;
varying vec3 vViewDirection;

void main() {
	vec3 normal = normalize(vNormal);
	vec3 viewDirection = normalize(vViewDirection);

	vec4 baseColor = texture2D(sBaseMap, vTexCoord);
	float diffuseValue = 0.5 * (dot(cLightNormal, normal) + 1.0);
	vec4 diffuse = baseColor * diffuseValue;

	vec2 irradPos = vWorldPosition.yx;
	vec2 irradUV = (irradPos - cIrradiancePos) / cIrradianceDim;
	float irradiance = texture2D(sIrradianceMap, irradUV).r;

	vec3 reflection = reflect(cLightNormal, normal);
	float rDotV = max(0.0, dot(reflection, viewDirection));
	float specularValue = pow(rDotV, cSpecularPower);
	vec4 specular = vec4(specularValue, specularValue, specularValue, 1.0);

	gl_FragColor = irradiance * (diffuse + specular);
}
